import re
import urllib.request
import click


def extract_unicode_ranges(text):
    pattern = r"^\s*unicode-range:\s+([^;]+);"
    matches = re.findall(pattern, text, flags=re.MULTILINE)
    matches = [i.upper() for i in matches]
    return matches


@click.command()
@click.option("--output_file", type=click.Path(), default="unicode_ranges.txt")
@click.option(
    "--css_url",
    type=str,
    default="https://fonts.googleapis.com/css2?family=Noto+Sans+JP",
)
@click.option(
    "--user_agent",
    type=str,
    default="Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Chrome/133.0.0.0",
)
def get_unicode_ranges_from_google_fonts(
    output_file: str, css_url: str, user_agent: str
):
    print(f"Download: {css_url}")
    req = urllib.request.Request(css_url, headers={"User-Agent": user_agent})

    with urllib.request.urlopen(req) as response:
        if response.getcode() != 200:
            raise (f"Failed to download: {css_url}")
        text = response.read().decode("utf-8")

    unicode_ranges = extract_unicode_ranges(text)
    with open(output_file, "w") as file:
        file.write("\n".join(unicode_ranges))
        print(f"Success to generate: {output_file}")



from http.server import HTTPServer
from http.server import BaseHTTPRequestHandler
from string import Template
import re
import os
import click


class CustomHTTPServer(HTTPServer):
    def __init__(self, server_address, RequestHandlerClass, html):
        super().__init__(server_address, RequestHandlerClass)
        self.html = html


class CustomHTTPRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == "/":
            self.send_response(200)
            self.send_header("Content-type", "text/html; charset=utf-8")
            self.end_headers()
            self.wfile.write(self.server.html.encode())
            return

        path = "./" + self.path[1:]
        if os.path.exists(path):
            self.send_response(200)
            if path.endswith(".css"):
                self.send_header("Content-type", "text/css; charset=utf-8")
            elif path.endswith(".woff2"):
                self.send_header("Content-type", "font/woff2")
            else:
                self.send_header("Content-type", "text/plain; charset=utf-8")
            self.end_headers()
            with open(path, "rb") as f:
                self.wfile.write(f.read())
            return

        self.send_response(404)
        self.end_headers()
        self.wfile.write(b"File Not Found")


TEMPLATE = Template(
    """\
<html lang="ja">
  <head>
    <title>fontsplitta test</title>
    <meta charset="UTF-8" />
    <link rel="stylesheet" type="text/css" href="${css_file}" />
    <style>
      body {
        font-family: "${font_family}";
        font-size: 48px;
      }
    </style>
  </head>
  <body>
    あのイーハトーヴォのすきとおった風、<br />
    夏でも底に冷たさをもつ青いそら、<br />
    うつくしい森で飾られたモリーオ市、<br />
    郊外のぎらぎらひかる草の波。
  </body>
</html>
"""
)


def generate_html(css_file: str):
    with open(css_file, "r") as file:
        content = file.read()
    pattern = r"^\s*font-family:\s+\"([^\"]+)\";$"
    font_family = re.search(pattern, content, flags=re.MULTILINE).group(1)

    return TEMPLATE.substitute(font_family=font_family, css_file=css_file)


@click.command()
@click.option("--addr", type=str, default="localhost")
@click.option("--port", type=int, default=8080)
@click.option("--css_file", type=str, default="output/font-face.css")
def test_server(addr: str, port: int, css_file: str):
    html = generate_html(css_file)
    with CustomHTTPServer((addr, port), CustomHTTPRequestHandler, html) as httpd:
        print(f"Serving on http://{addr}:{port}/")
        httpd.serve_forever()



from fontTools.subset import main as pyftsubset
from fontTools import ttLib
from string import Template
import hashlib
import base64
import click
import os

CSS_TEMPLATE = Template(
    """\
@font-face {
  font-family: "${font_family}";
  font-style: ${font_style};
  font-weight: ${font_weight};
  src: url(http://localhost:8080/output/${font_filename}) format("${font_format}");
  unicode-range: ${unicode_range};
}
"""
)


def get_font_info(fontPath: str) -> dict:
    font = ttLib.TTFont(fontPath)
    return {
        "font_family": font["name"].names[1],
        "font_weight": font["OS/2"].usWeightClass,
    }


def unique_filename(unicode_range: str) -> str:
    return (
        base64.urlsafe_b64encode(hashlib.sha1(unicode_range.encode()).digest())
        .decode("utf-8")
        .replace("=", "")
    )


def generate_subset_font(
    font_file: str, unicode_range: str, font_format: str, output_dir: str, force: bool
) -> str:
    if not os.path.exists(output_dir):
        os.mkdir(output_dir)

    filename = ("{}.{}").format(unique_filename(unicode_range), font_format)
    output_file = ("{}/{}").format(output_dir, filename)

    if not force and os.path.exists(output_file):
        print("[INFO] skip: " + output_file)
        return filename

    pyftsubset(
        [
            font_file,
            "--unicodes=" + unicode_range,
            "--layout-features=*",
            "--flavor=" + font_format,
            "--output-file=" + output_file,
        ]
    )
    print("[INFO] generate: " + output_file)
    return filename


@click.command()
@click.argument("font_file", type=click.Path(exists=True))
@click.option(
    "--unicode_ranges_file", type=click.Path(exists=True), default="unicode_ranges.txt"
)
@click.option(
    "--css_template_file",
    type=click.Path(exists=True),
)
@click.option("--output_dir", type=click.Path(), default="output")
@click.option("--font_format", type=click.Choice(["woff2", "woff"]), default="woff2")
@click.option("--font_face_css", type=click.Path(), default="font-face.css")
@click.option("--force", is_flag=True, default=False)
def split(
    font_file: str,
    unicode_ranges_file: str,
    css_template_file: str,
    output_dir: str,
    font_format: str,
    font_face_css: str,
    force: bool,
):
    if not os.path.exists(font_file):
        print("[ERROR] font file not found: " + font_file)
        return

    if not os.path.exists(unicode_ranges_file):
        print("[ERROR] unicode ranges file not found: " + unicode_ranges_file)
        return

    with open(unicode_ranges_file, "r") as file:
        content = file.read()
        unicode_ranges = content.split("\n")

    if unicode_ranges == [""]:
        print("[ERROR] No unicode ranges found in the file")
        return

    print("[INFO] unicode-range: {} ranges found".format(len(unicode_ranges)))

    if css_template_file and os.path.exists(css_template_file):
        print("[INFO] Using custom css template: " + css_template_file)
        with open(css_template_file, "r") as file:
            css_template = Template(file.read())
    else:
        print("[INFO] Using default css template")
        css_template = CSS_TEMPLATE

    font_info = get_font_info(font_file)
    font_family = str(font_info["font_family"])
    font_weight = str(font_info["font_weight"])

    print("[INFO] font-family: " + font_family + ", font-weight: " + font_weight)

    font_faces = []
    for unicode_range in unicode_ranges:
        subset_font_filename = generate_subset_font(
            font_file, unicode_range, font_format, output_dir, force
        )

        font_faces.append(
            css_template.substitute(
                font_family=font_family,
                font_style="normal",
                font_weight=font_weight,
                font_filename=subset_font_filename,
                font_format=font_format,
                unicode_range=unicode_range,
            )
        )

    print("[INFO] generate CSS: " + output_dir + "/" + font_face_css)
    with open(output_dir + "/" + font_face_css, "w") as file:
        file.write("\n".join(font_faces))

    print("[INFO] success to generate")



import click
from fontsplitta import split
from fontsplitta import test_server
from fontsplitta import google_fonts

from importlib.metadata import version

__version__ = version("fontsplitta")


@click.group()
@click.version_option(version=__version__)
def cli():
    pass


def main():
    cli.add_command(split.split)
    cli.add_command(test_server.test_server)
    cli.add_command(google_fonts.get_unicode_ranges_from_google_fonts)
    cli()


if __name__ == "__main__":
    main()
